"""Ship Type Model."""


class ShipType(object):
    """Ship Type.

    from data shiptype.json

    """

    def __init__(self, type_id, name, chinese_name, english_name):
        """Initialize the instance variables.

        Args:
            type_id (int): The type id
            name (str): The name
            chinese_name (str): The Chinese name
            english_name (str): The English name
        """
        self.id = type_id  # The type id
        self.name = name  # The name
        self.chinese_name = chinese_name  # The Chinese name
        self.english_name = english_name  # The English name


def load_ship_type(json) -> ShipType:
    """Load ship type.

    Args:
        json (dict): json

    Returns:
        ShipType:

    """
    return ShipType(
        type_id=json.get("id"),
        name=json.get("name"),
        chinese_name=json.get("chinese_name"),
        english_name=json.get("english_name"),
    )


def load_ship_type_list(json_list) -> list[ShipType]:
    """Load ship type list.

    Args:
        json_list (list[dict]): json list

    Returns:
        list[ShipType]:

    """
    return [load_ship_type(json) for json in json_list]



"""Data module."""
import json
from os import path

import requests  # type: ignore

from kancolle.data import KC_DATA_URL, MODULE_PATH

session = requests.Session()


def download():
    """Download ship, ship type, and ship class data and save to JSON."""
    file_data = {
        "ship.json": "ship/ship.json",
        "shiptype.json": "shiptype/all.json",
        "shipclass.json": "shipclass/all.json",
        "slotitem.json": "slotitem/all.json",
    }

    for filename, _path in file_data.items():
        url = KC_DATA_URL + _path
        data = session.get(url).json()
        save_data(data, filename)


def save_data(data, file_name):
    """This function saves a given data into a specified file.

    Args:
        data (dict): The data to be saved. It should be a dictionary.
        file_name (str): The name of the file where the data will be saved.

    Returns:
        None
    """
    file_name = path.join(MODULE_PATH, file_name)
    with open(file_name, 'w', encoding="utf-8") as f:
        f.write(json.dumps(data, ensure_ascii=False, indent=4))


def load_data(file_name):
    """Load data from file.

    Args:
        file_name (str): file name

    Returns:
        dict
    """
    file_name = path.join(MODULE_PATH, file_name)
    with open(file_name, 'r', encoding="utf-8") as f:
        return json.loads(f.read())


def load_ship_data():
    """Load ship data."""
    return load_data("ship.json")


def load_ship_type_data():
    """Load ship type data."""
    return load_data("shiptype.json")


def load_ship_class_data():
    """Load ship class data."""
    return load_data("shipclass.json")


def load_slot_item_data():
    """Load slotitem data."""
    return load_data("slotitem.json")


if __name__ == '__main__':
    download()



"""Main module."""
from kancolle.data import data
from kancolle.models import ship, ship_class, ship_type, slot_item


def get_ships():
    """Get all ships."""
    return [ship.load_ship(_data) for _data in data.load_ship_data()]


def get_ship_types():
    """Get all ship types."""
    return [ship_type.load_ship_type(_data) for _data in data.load_ship_type_data()]


def get_ship_classes():
    """Get all ship classes."""
    return [ship_class.load_ship_class(_data) for _data in data.load_ship_class_data()]


def get_slot_items():
    """Get all slot items."""
    return [slot_item.load_slot_item(_data) for _data in data.load_slot_item_data()]


class Kancolle:
    """Kancolle class."""

    def __init__(self):
        """Initialize."""
        self.ships = get_ships()
        self.ship_types = get_ship_types()
        self.ship_classes = get_ship_classes()
        self.slot_items = get_slot_items()



"""Console script for kancolle."""

import click


@click.command()
def main():
    """Main entrypoint."""
    click.echo("kancolle")
    click.echo("=" * len("kancolle"))
    click.echo("for KanColle assistant software")


if __name__ == "__main__":
    main()  # pragma: no cover



"""Slot Item Model."""


class SlotItem:
    """Slot Item Model."""

    def __init__(self, item_id, name, type_id_list) -> None:
        """Initializes an instance of the class.

        Args:
            item_id (int): The ID of the slot item.
            name (str): The name of the slot item.
            type_id_list (list): The type ID list of the slot item.

        Returns:
            None
        """
        self.id = item_id
        self.name = name
        self.type_id_list = type_id_list


def load_slot_item(json) -> SlotItem:
    """Load slot item.

    Args:
        json (dict): json

    Returns:
        SlotItem:

    """
    return SlotItem(item_id=json.get("id"), name=json.get("name"), type_id_list=json.get("type"))


def load_slot_item_list(json_list) -> list[SlotItem]:
    """Load slot item list.

    Args:
        json_list (list[dict]): json list

    Returns:
        list[SlotItem]:

    """
    return [load_slot_item(json) for json in json_list]



"""Data module."""
import importlib.resources

MODULE_PATH = importlib.resources.files(__package__)
# only on python >= 3.9, see https://docs.python.org/3/library/importlib.resources.html
# MODULE_PATH = os.path.abspath(os.path.dirname(__file__)) for python < 3.9
KC_DATA_URL = "https://andychucs.github.io/kcwiki-kcdata/"
KC_QUEST_URL = "https://andychucs.github.io/kcQuest/"



"""Ship model."""


class Ship(object):
    """Ship model.

    from data ship.json

    """

    def __init__(self, ship_id, name, class_id, type_id) -> None:
        """Initializes an instance of the class.

        Args:
            ship_id (int): The ID of the ship.
            name (str): The name of the ship.
            class_id (int): The class ID of the ship.
            type_id (int): The type ID of the ship.

        Returns:
            None
        """
        self.id = ship_id
        self.name = name
        self.class_id = class_id
        self.type_id = type_id


def load_ship(json) -> Ship:
    """Load ship from json.

    Args:
        json (dict): ship json data dict

    Returns:
        Ship: ship

    """
    return Ship(ship_id=json.get('id'), name=json.get('name'), class_id=json.get('ctype'), type_id=json.get('stype'))


def load_ship_list(json_list) -> list[Ship]:
    """Load ships from json list.

    Args:
        json_list (list[dict]): ship json list
    Returns:
        list[Ship]: ships
    """
    return [load_ship(json) for json in json_list]



"""Ship Class."""


class ShipClass(object):
    """Ship Class.

    from data shipclass.json: {"id": 1,"name": "綾波型","chinese_name": "绫波型"}


    """

    def __init__(self, class_id, name, chinese_name):
        """Initializes the instance with the specified class_id, name, and chinese_name. It doesn't return any value.

        Args:
            class_id (int): The ID for the class.
            name (str): The English name for the class.
            chinese_name (str): The Chinese name for the class.
        """
        self.id = class_id
        self.name = name
        self.chinese_name = chinese_name


def load_ship_class(json) -> ShipClass:
    """Load Ship Class from json.

    Args:
        json (dict):

    Returns:
        ShipClass

    """
    return ShipClass(class_id=json.get('id'), name=json.get('name'), chinese_name=json.get('chinese_name'))


def load_ship_class_list(json_list) -> list[ShipClass]:
    """Load Ship Class from json list.

    Args:
        json_list (list[dict]):

    Returns:
        list[ShipClass]

    """
    return [load_ship_class(json) for json in json_list]



#!/usr/bin/env python
"""Tests for `kancolle` package."""

import pytest
from click.testing import CliRunner

from kancolle import cli


@pytest.fixture
def ship_data_response():
    """Ship data response."""
    from kancolle.data import KC_DATA_URL

    ship_data_url = KC_DATA_URL + "ship/ship.json"
    import requests  # type: ignore

    return requests.get(ship_data_url).json()


@pytest.fixture
def ship_type_response():
    """Ship type response."""
    from kancolle.data import KC_DATA_URL

    ship_type_url = KC_DATA_URL + "shiptype/all.json"
    import requests  # type: ignore

    return requests.get(ship_type_url).json()


@pytest.fixture
def ship_class_response():
    """Ship class response."""
    from kancolle.data import KC_DATA_URL

    ship_class_url = KC_DATA_URL + "shipclass/all.json"
    import requests  # type: ignore

    return requests.get(ship_class_url).json()


def test_load_ship_data(ship_data_response, ship_class_response, ship_type_response):
    """Test load ship data."""
    from kancolle.models import ship
    from kancolle.models import ship_class
    from kancolle.models import ship_type

    ships = ship.load_ship_list(ship_data_response)
    ship_classes = ship_class.load_ship_class_list(ship_class_response)
    ship_types = ship_type.load_ship_type_list(ship_type_response)

    assert ships[0].name == '睦月'

    class_name = next((sc.name for sc in ship_classes if sc.id == ships[0].class_id), '')

    assert ships[0].name in class_name

    type_name = next((st.name for st in ship_types if st.id == ships[0].type_id), '')

    assert type_name == "駆逐艦"


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'kancolle' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output


def test_data_download():
    """Test data download."""
    from kancolle.data import data

    data.download()

    assert type(data.load_ship_data()) == list
    assert type(data.load_ship_class_data()) == list
    assert type(data.load_ship_type_data()) == list


def test_data_load():
    """Test data load."""
    from kancolle.data import data
    from kancolle.models import ship, ship_type, ship_class, slot_item

    assert type(data.load_ship_data()) == list
    assert type(data.load_ship_class_data()) == list
    assert type(data.load_ship_type_data()) == list

    ships = ship.load_ship_list(data.load_ship_data())
    ship_classes = ship_class.load_ship_class_list(data.load_ship_class_data())
    ship_types = ship_type.load_ship_type_list(data.load_ship_type_data())

    assert ships[0].name == '睦月'

    class_name = next((sc.name for sc in ship_classes if sc.id == ships[0].class_id), '')

    assert ships[0].name in class_name

    type_name = next((st.name for st in ship_types if st.id == ships[0].type_id), '')

    assert type_name == "駆逐艦"

    assert type(data.load_slot_item_data()) == list

    slot_items = slot_item.load_slot_item_list(data.load_slot_item_data())

    assert slot_items[0].name == '12cm単装砲'


def test_kancolle_class():
    """Test Kancolle class."""
    import kancolle.kancolle as kc

    kancolle_data = kc.Kancolle()

    assert kancolle_data.ships[0].name == "睦月"
    assert kancolle_data.slot_items[0].name == "12cm単装砲"
